!{\src2tex{textfont=tt}}
!!****f* ABINIT/kgindex
!! NAME
!! kgindex
!!
!! FUNCTION
!! Compute the index of each plane wave on a FFT grid.
!!
!! COPYRIGHT
!! Copyright (C) 2001 ABINIT group (XG)
!! This file is distributed under the terms of the
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!!
!! INPUTS
!!  iind=shift to be given to the location of the output data in the array indpw
!!  ikg=shift to be given to the location of the output data in the array kg
!!  kg(3,npw)=dimensionless coords of G vecs (integer)
!!  mgfft=maximum size of 1D FFTs
!!  mkmem =maximum number of k points which can fit in core memory
!!  mpw=maximum number of planewaves as dimensioned in calling routine
!!  ngfft(1:3)=integer fft box dimensions, see getng for ngfft(4:8)
!!  npw=number of planewaves inside ecut centered at kpt
!!
!! OUTPUT
!!  indpw(npw)=linear list number (in fft box) of given G vector
!!
!! SIDE EFFECTS
!!
!! NOTES
!!
!! PARENTS
!!      kpgio,wfconv
!!
!! SOURCE

 subroutine kgindex(iind,ikg,indpw,&
&   kg,mgfft,mkmem,mpw,ngfft,npw)

 use basis_defs
 implicit none

!Arguments ------------------------------------
 integer :: iind,ikg,mgfft,mkmem,mpw,npw
 integer :: indpw((mpw+(2*mgfft+4))*mkmem),&
&    kg(3,mpw*mkmem),ngfft(8)

!Local variables-------------------------------
 integer :: ig,ig1,ig2,ig3
 
! *************************************************************************
 
!DEBUG
!write(6,*)' kgindex : enter '
!ENDDEBUG

!Use the following indexing (N means ngfft of the adequate direction)
!0 1 2 3 ... N/2    -(N-1)/2 ... -1    <= kg
!1 2 3 4 ....N/2+1  N/2+2    ...  N    <= index

 do ig=1,npw
  ig1=kg(1,ig+ikg)+1 ; if(ig1<1)ig1=ig1+ngfft(1)
  ig2=kg(2,ig+ikg)+1 ; if(ig2<1)ig2=ig2+ngfft(2)
  ig3=kg(3,ig+ikg)+1 ; if(ig3<1)ig3=ig3+ngfft(3)
  indpw(ig+iind)=ig1+ngfft(1)*((ig2-1)+ngfft(2)*(ig3-1))
 enddo

!DEBUG
!write(6,*)' kgindex : exit'
!stop
!ENDDEBUG

 end subroutine
!!***
