!{\src2tex{textfont=tt}}
!!****f* ABINIT/sphere
!!
!! NAME
!! sphere 
!!
!! FUNCTION
!! Array cg is defined in sphere with npw points. Insert cg inside box
!! of n1*n2*n3 points to define array cfft for fft box.
!! indpw tells what index in cfft
!! corresponds to given element in cg.  rest of cfft is filled with 0 s.
!! iflag=1==>insert cg into cfft.  iflag=-1==> extract cg from cfft.
!! There is also the possibility to apply a symmetry operation,
!! as well as to make a shift in reciprocal space, in the case iflag=-1.
!!
!! COPYRIGHT
!! Copyright (C) 1998-2001 ABINIT group (DCA, XG, GMR, AR)
!! This file is distributed under the terms of the 
!! GNU General Public License, see ~ABINIT/Infos/copyright
!! or http://www.gnu.org/copyleft/gpl.txt .
!! For the initials of contributors, see ~ABINIT/Infos/contributors .
!!
!!
!! NOTES
!! cg and cfft are assumed to be of type COMPLEX, although this routine treats
!! them as real of twice the length to avoid nonstandard complex*16.
!! If istwf_k differs from 1, then special storage modes must be taken
!! into account, for symmetric wavefunctions coming from k=(0 0 0) or other
!! special k points.
!!
!! WARNING
!! Inputs and outputs description missing
!!
!! TODO
!! Order arguments
!! 
!! PARENTS
!!      fourwf,wfconv
!!
!! SOURCE

 subroutine sphere(cg,npw,cfft,n1,n2,n3,n4,n5,n6,indpw_k,istwf_k,iflag,&
& shiftg,symm)

 use basis_defs
 implicit none

!Arguments ------------------------------------
 integer :: iflag,istwf_k,npw,n1,n2,n3,n4,n5,n6
 integer :: indpw_k(4,npw),shiftg(3),symm(3,3)
 real(dp) :: cg(2,npw),cfft(2,n4,n5,n6)

!Local variables-------------------------------
 integer :: ifft,ifftinv,ipw,istwf_kold,id1,id2,id3,&
& i1,i1inv,i2,i2inv,i3,i3inv,&
& j1,j2,j3,l1,l2,l3,npwmin,use_symmetry
 integer :: identity(3,3)
 integer, allocatable :: i1inver(:),i2inver(:),i3inver(:)
 character*500 :: message
 
! *************************************************************************
 
!call timab(18,1,tsec)

!In the case of special k-points, invariant under time-reversal,
!but not Gamma, initialize the inverse coordinates
!DEBUG
!if(iflag==-1)then
! istwf_kold=istwf_k
! istwf_k=2
!endif
!ENDDEBUG
 if(istwf_k>=2)then
  allocate(i1inver(n1),i2inver(n2),i3inver(n3))
  if(istwf_k==2 .or. istwf_k==4 .or. istwf_k==6 .or. istwf_k==8)then
   i1inver(1)=1
   do i1=2,n1
    i1inver(i1)=n1+2-i1
   enddo
  else
   do i1=1,n1
    i1inver(i1)=n1+1-i1
   enddo
  endif
  if(istwf_k>=2 .and. istwf_k<=5)then
   i2inver(1)=1
   do i2=2,n2
    i2inver(i2)=n2+2-i2
   enddo
  else
   do i2=1,n2
    i2inver(i2)=n2+1-i2
   enddo
  endif
  if(istwf_k==2 .or. istwf_k==3 .or. istwf_k==6 .or. istwf_k==7)then
   i3inver(1)=1
   do i3=2,n3
    i3inver(i3)=n3+2-i3
   enddo
  else
   do i3=1,n3
    i3inver(i3)=n3+1-i3
   enddo
  endif
 endif

!Begin main part here
 if (iflag==1) then 

! Insert cg into cfft with extra 0 s around outside:
!$OMP PARALLEL DO PRIVATE(i1,i2,i3) SHARED(cfft,n1,n2,n3)
  do i3=1,n3
   do i2=1,n2
    do i1=1,n1
     cfft(1,i1,i2,i3)=0.0d0
     cfft(2,i1,i2,i3)=0.0d0
    enddo
   enddo
  enddo
!$OMP END PARALLEL DO

! Take care of each plane wave, and complete cfft if needed

  if(istwf_k==1)then

!$OMP PARALLEL DO PRIVATE(i1,i2,i3,ipw) SHARED(cfft,cg,indpw_k,npw)
   do ipw=1,npw
    i1=indpw_k(1,ipw) ; i2=indpw_k(2,ipw) ; i3=indpw_k(3,ipw)
    cfft(1,i1,i2,i3)=cg(1,ipw)
    cfft(2,i1,i2,i3)=cg(2,ipw)
   enddo
!$OMP END PARALLEL DO

  else if(istwf_k>=2)then

   npwmin=1
   if(istwf_k==2)then
!   If wavefunction comes from gamma point, then cfft must be completed
    cfft(1,1,1,1)=cg(1,1)
    cfft(2,1,1,1)=0.0d0
    npwmin=2
   endif

!$OMP PARALLEL DO PRIVATE(i1,i1inv,i2,i2inv,i3,i3inv,ipw) &
!$OMP&SHARED(cfft,cg,i1inver,i2inver,i3inver,indpw_k,npw,npwmin)
#if defined nec
!CDIR NODEP
#endif
   do ipw=npwmin,npw

    i1=indpw_k(1,ipw) ; i2=indpw_k(2,ipw) ; i3=indpw_k(3,ipw)
!   Construct the coordinates of -k-G
    i1inv=i1inver(i1) ; i2inv=i2inver(i2) ; i3inv=i3inver(i3)

    cfft(1,i1,i2,i3)=cg(1,ipw)
    cfft(2,i1,i2,i3)=cg(2,ipw)
    cfft(1,i1inv,i2inv,i3inv)= cg(1,ipw)
    cfft(2,i1inv,i2inv,i3inv)=-cg(2,ipw)
!
   enddo
!$OMP END PARALLEL DO

  endif

 else if (iflag==-1) then 

  use_symmetry=0
  identity(:,:)=0
  identity(1,1)=1 ; identity(2,2)=1 ; identity(3,3)=1
  if(sum((symm(:,:)-identity(:,:))**2)/=0)use_symmetry=1
  if(sum(shiftg(:)**2)/=0)use_symmetry=1
! Extract cg from cfft, ignoring components outside range of cg:
  if(istwf_k==1)then

   if(use_symmetry==0)then
!$OMP PARALLEL DO PRIVATE(i1,i2,i3,ipw) SHARED(cfft,cg,indpw_k,npw)
    do ipw=1,npw
     i1=indpw_k(1,ipw) ; i2=indpw_k(2,ipw) ; i3=indpw_k(3,ipw)
     cg(1,ipw)=cfft(1,i1,i2,i3)
     cg(2,ipw)=cfft(2,i1,i2,i3)
    enddo
!$OMP END PARALLEL DO
   else 
    id1=n1/2+2
    id2=n2/2+2
    id3=n3/2+2
!$OMP PARALLEL DO PRIVATE(i1,i2,i3,j1,j2,j3,l1,l2,l3,ipw)&
!$OMP&SHARED(cfft,cg,id1,id2,id3,indpw_k,npw,n1,n2,n3,shiftg,symm)
    do ipw=1,npw
     i1=indpw_k(1,ipw) ; i2=indpw_k(2,ipw) ; i3=indpw_k(3,ipw)
     l1=i1-(i1/id1)*n1-1+shiftg(1)
     l2=i2-(i2/id2)*n2-1+shiftg(2)
     l3=i3-(i3/id3)*n3-1+shiftg(3)
     j1=symm(1,1)*l1+symm(1,2)*l2+symm(1,3)*l3
     j2=symm(2,1)*l1+symm(2,2)*l2+symm(2,3)*l3
     j3=symm(3,1)*l1+symm(3,2)*l2+symm(3,3)*l3
     if(j1<0)j1=j1+n1 ; i1=j1+1
     if(j2<0)j2=j2+n2 ; i2=j2+1
     if(j3<0)j3=j3+n3 ; i3=j3+1
     cg(1,ipw)=cfft(1,i1,i2,i3)
     cg(2,ipw)=cfft(2,i1,i2,i3)
    enddo
!$OMP END PARALLEL DO
   endif

  else if(istwf_k>=2)then

   npwmin=1
   if(istwf_k==2)then
!   Extract cg from cfft, in a way that project on a wavefunction
!   with time-reversal symmetry
    cg(1,1)=cfft(1,1,1,1)
    cg(2,1)=0.0d0
    npwmin=2
   endif

   if(use_symmetry==0)then
!$OMP PARALLEL DO PRIVATE(i1,i1inv,i2,i2inv,i3,i3inv,ipw) &
!$OMP&SHARED(cfft,cg,i1inver,i2inver,i3inver,indpw_k,npw,npwmin)
    do ipw=npwmin,npw
     i1=indpw_k(1,ipw) ; i2=indpw_k(2,ipw) ; i3=indpw_k(3,ipw)
!    Construct the coordinates of -k-G
     i1inv=i1inver(i1) ; i2inv=i2inver(i2) ; i3inv=i3inver(i3)
 
!    Here the time-reversal symmetry is used to project from cfft
     cg(1,ipw)=(cfft(1,i1,i2,i3)+ &
&               cfft(1,i1inv,i2inv,i3inv))*0.5d0
     cg(2,ipw)=(cfft(2,i1,i2,i3)- &
&               cfft(2,i1inv,i2inv,i3inv))*0.5d0
 
    enddo
!$OMP END PARALLEL DO
   else
    id1=n1/2+2
    id2=n2/2+2
    id3=n3/2+2

!$OMP PARALLEL DO PRIVATE(i1,i1inv,i2,i2inv,i3,i3inv,ipw) &
!$OMP&PRIVATE(j1,j2,j3,l1,l2,l3)&
!$OMP&SHARED(cfft,cg,id1,id2,id3,indpw_k)&
!$OMP&SHARED(i1inver,i2inver,i3inver,npw,npwmin,n1,n2,n3,shiftg,symm)
    do ipw=npwmin,npw

     i1=indpw_k(1,ipw) ; i2=indpw_k(2,ipw) ; i3=indpw_k(3,ipw)
     i1inv=i1inver(i1) ; i2inv=i2inver(i2) ; i3inv=i3inver(i3)

     l1=i1-(i1/id1)*n1-1+shiftg(1)
     l2=i2-(i2/id2)*n2-1+shiftg(2)
     l3=i3-(i3/id3)*n3-1+shiftg(3)
     j1=symm(1,1)*l1+symm(1,2)*l2+symm(1,3)*l3
     j2=symm(2,1)*l1+symm(2,2)*l2+symm(2,3)*l3
     j3=symm(3,1)*l1+symm(3,2)*l2+symm(3,3)*l3
     if(j1<0)j1=j1+n1 ; i1=j1+1
     if(j2<0)j2=j2+n2 ; i2=j2+1
     if(j3<0)j3=j3+n3 ; i3=j3+1
     
!    Construct the coordinates of -k-G
     l1=i1inv-(i1inv/id1)*n1-1+shiftg(1)
     l2=i2inv-(i2inv/id2)*n2-1+shiftg(2)
     l3=i3inv-(i3inv/id3)*n3-1+shiftg(3)
     j1=symm(1,1)*l1+symm(1,2)*l2+symm(1,3)*l3
     j2=symm(2,1)*l1+symm(2,2)*l2+symm(2,3)*l3
     j3=symm(3,1)*l1+symm(3,2)*l2+symm(3,3)*l3
     if(j1<0)j1=j1+n1 ; i1inv=j1+1
     if(j2<0)j2=j2+n2 ; i2inv=j2+1
     if(j3<0)j3=j3+n3 ; i3inv=j3+1

!    Here the time-reversal symmetry is used to project from cfft
     cg(1,ipw)=(cfft(1,i1,i2,i3)+ &
&               cfft(1,i1inv,i2inv,i3inv))*0.5d0
     cg(2,ipw)=(cfft(2,i1,i2,i3)- &
&               cfft(2,i1inv,i2inv,i3inv))*0.5d0
    enddo
!$OMP END PARALLEL DO
   endif

  endif

 else
  write(message, '(a,a,a,a,i10,a)' ) ch10,&
&   'sphere : BUG -',ch10,&
&   '  iflag=',iflag,' not acceptable.'
  call wrtout(06,message,'PERS')
  call leave_new('PERS')
 end if 

 if(istwf_k>=2)then
  deallocate(i1inver,i2inver,i3inver)
 endif

!DEBUG
!if(iflag==-1)then
! write(6,*)'sphere : cg(:,:)'
! do ipw=1,npw
!  write(6, '(4i6,2es16.6)') ipw,indpw_k(1:3,ipw),cg(1:2,ipw)
! enddo
!endif
!ENDDEBUG

!DEBUG
!if(iflag==-1)then
! istwf_k=istwf_kold
!endif
!ENDDEBUG
 
!call timab(18,2,tsec)

 end subroutine
!!***
