
#include "partition.h"
#include "fxttypes.h"

#include "fxtio.h"
#include "fxtiomanip.h"


void
partition::init(ulong x)
// reset state
{
    x_ = x;
    ct_ = 0;
    for (ulong k=0; k<n_; ++k)  pc_[k] = 0;
    for (ulong k=0; k<n_; ++k)  r_[k] = 0;
    r_[n_-1] = x;
    r_[n_] = x;
    i_ = n_ - 1;
    pci_ = 0;
    ri_ = x;
}
// -------------------------


int
partition::check(ulong i/*=0*/)  const
{
    long xx = 0;
    for (ulong k=i;  k<n_; ++k)  xx += pc_[k]*pv_[k];
    return  (xx==x_);
}
// -------------------------


void
partition::dump()  const
{
//    cout << "{" << i << "} ";
    cout.width(4);  cout << x_;  cout << " == ";

    for (ulong k=0; k<n_; ++k)
    {
        cout << " +";  cout.width(2);  cout << pc_[k];
        if ( 0!=pc_[k] )  { cout << "*";  cout.width(2);  cout << pv_[k]; }
        else              cout << "   ";

//        cout << " [";  cout.width(2);  cout << r_[k] << "]";
    }

//    cout << "  [[";  cout.width(2);  cout << r_[n_] << "]]";  // == const = x
//    if ( 0==check() )  cout << "  ****** ";

    cout << endl;
}
// -------------------------


ulong
partition::next()
// generate next partition
{
    if ( i_>=n_ )  return  n_;

    r_[i_] = ri_;
    pc_[i_] = pci_;
    i_ = next_func(i_);

    for (ulong j=0; j<i_; ++j)  pc_[j] = r_[j] = 0;

    ++i_;
    ri_ = r_[i_] - pv_[i_];
    pci_ = pc_[i_] + 1;

    return  i_ - 1;  // >=0
}
// -------------------------

ulong
partition::next_func(ulong i)
{
 start:
    if ( 0!=i )
    {
        while ( r_[i]>0 )
        {
            pc_[i-1] = 0;
            r_[i-1] = r_[i];
            --i;  goto start;   // iteration
        }
    }
    else  // iteration end
    {
        if ( 0!=r_[i] )
        {
            long d = r_[i] / pv_[i];
            r_[i] -= d * pv_[i];
            pc_[i] = d;
        }
    }

    if ( 0==r_[i] )  // valid partition found
    {
        ++ct_;
        return i;
    }

    ++i;
    if ( i>=n_ )  return n_;  // search finished

    r_[i] -= pv_[i];
    ++pc_[i];

    goto start;  // iteration
}
// -------------------------


ulong
partition::count(ulong x)
// count number of partitions
{
    init(x);
    count_func(n_-1);
    return ct_;
}
// -------------------------

ulong
partition::count_func(ulong i)
{
    if ( 0!=i )
    {
        while ( r_[i]>0 )
        {
            pc_[i-1] = 0;
            r_[i-1] = r_[i];
            count_func(i-1);  // recursion
            r_[i] -= pv_[i];
            ++pc_[i];
        }
    }
    else  // recursion end
    {
        if ( 0!=r_[i] )
        {
            long d = r_[i] / pv_[i];
            r_[i] -= d * pv_[i];
            pc_[i] = d;
        }
    }

    if ( 0==r_[i] )  // valid partition found
    {
        // if ( whatever )  ++ct_;  // restricted count
        ++ct_;
        return 1;
    }
    else  return 0;
}
// -------------------------
