
#include "fxttypes.h"
#include "fxt.h" // fourier_shift()


#include "copy.h"
#define copy fxtaux::copy // avoid unwanted std::copy


static void
recursive_fft_dit2_core(const Complex *a, ulong n, Complex *x, int is)
{
    if ( n==1 )  { x[0] = a[0]; return; }

    ulong nh = n/2;
    Complex b[nh];
    Complex c[nh];
    Complex ev[nh];
    Complex od[nh];

    for (ulong k=0,j=0; j<nh; k+=2,j++)
    {
        ev[j] = a[k];
        od[j] = a[k+1];
    }

    recursive_fft_dit2_core(ev, nh, b, is);
    recursive_fft_dit2_core(od, nh, c, is);

    fourier_shift(c, nh, is*0.5);

    for (ulong k=0; k<nh; k++)
    {
        x[k]    = b[k] + c[k];
        x[k+nh] = b[k] - c[k];
    }
}
// -------------------------

void
recursive_fft_dit2(Complex *a, ulong ldn, int is)
//
// very inefficient, just to demonstrate the
// recursive fast fourier transform
//
{
    ulong n = (1UL<<ldn);
    Complex x[n];
    recursive_fft_dit2_core(a, n, x, is);
    copy(x, a, n);
}
// -------------------------


static void
recursive_fft_dif2_core(const Complex *a, ulong n, Complex *x, int is)
{
    if ( n==1 )  { x[0] = a[0]; return; }

    ulong nh = n/2;
    Complex b[nh];
    Complex c[nh];
    Complex le[nh];
    Complex ri[nh];

    for (ulong j=0; j<nh; j++)
    {
        le[j] = a[j];
        ri[j] = a[nh+j];
    }

    for (ulong k=0; k<nh; k++)
    {
        Complex t =   (le[k] + ri[k]);
        ri[k] =       (le[k] - ri[k]);
        le[k] = t;
    }

    fourier_shift(ri, nh, is*0.5);

    recursive_fft_dif2_core(le, nh, b, is);
    recursive_fft_dif2_core(ri, nh, c, is);

    for (ulong k=0, j=0; k<nh; k++,j+=2)
    {
        x[j] =   b[k];
        x[j+1] = c[k];
    }
}
// -------------------------

void
recursive_fft_dif2(Complex *a, ulong ldn, int is)
//
// very inefficient, just to demonstrate the
// recursive fast fourier transform
//
{
    ulong n = (1UL<<ldn);
    Complex x[n];

    recursive_fft_dif2_core(a, n, x, is);
    copy(x, a, n);
}
// -------------------------

