#if !defined HAVE_SEARCH_H__
#define      HAVE_SEARCH_H__


#include "fxttypes.h"


template <typename Type>
ulong bsearch(const Type *f, ulong n, const Type v)
//
// return index of first element in f[] that is == v
// return ~0 if there is no such element
// f[] must be sorted in ascending order
//
// must have  n!=0
{
    ulong nlo=0, nhi=n-1;
    while ( nlo != nhi )
    {
        ulong t = (nhi+nlo)/2;

        if ( f[t] < v )  nlo = t + 1;
        else             nhi = t;
    }

    if ( f[nhi]==v )  return nhi;
    else              return ~0UL;
}
// -------------------------


template <typename Type>
ulong bsearch_ge(const Type *f, ulong n, const Type v)
//
// return index of first element in f[] that is >= v
// return ~0 if there is no such element
// f[] must be sorted in ascending order
//
// must have  n!=0
{
    ulong nlo=0, nhi=n-1;
    while ( nlo != nhi )
    {
        ulong t = (nhi+nlo)/2;

        if ( f[t] < v )  nlo = t + 1;
        else             nhi = t;
    }

    if ( f[nhi]>=v )  return nhi;
    else              return ~0UL;
}
// -------------------------


template <typename Type>
ulong bsearch_le(const Type *f, ulong n, const Type v)
//
// return index of first element in f[] that is <= v
// return ~0 if there is no such element
// f[] must be sorted in ascending order
//
// must have  n!=0
{
    ulong nlo=0, nhi=n-1;
    while ( nlo != nhi )
    {
        ulong t = (nhi+nlo)/2;

        if ( f[t] > v )  nlo = t + 1;
        else             nhi = t;
    }

    if ( f[nhi]<=v )  return nhi;
    else              return ~0UL;
}
// -------------------------



template <typename Type>
ulong bsearch_approx(const Type *f, ulong n, const Type v, Type da)
//
// return index of first element x in f[] for which  |(x-v)| <= da
// return ~0 if there is no such element
// f[] must be sorted in ascending order
// da must be positive
// makes sense only with inexact types (float or double)
//
// must have  n!=0
{
    ulong k = bsearch_ge(f, n, v-da);
    if ( k<n )  k = bsearch_le(f+k, n-k, v+da);
    return k;
}
// -------------------------


template <typename Type>
inline long search_down(const Type *f, const Type v, ulong &i)
// search v in f[], starting at i (so i must be < length)
// f[i] must be greater or equal v
// f[] must be sorted in ascending order
// returns index k if f[k]==v or ~0 if no such k is found
// i is updated so that it can be used for a following
// search for an element u where u < v
{
    while ( (f[i]>v) && (i>0) )  --i;

    if ( f[i]==v )  return i;
    else            return ~0UL;
}
// -------------------------



#endif // !defined HAVE_SEARCH_H__
