/*     
 * Copyright (c) 2000 Carnegie Mellon University
 * Copyright (c) 1997-1999 Massachusetts Institute of Technology
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,MA  02111-1307 USA
 * 
 */

/* fftw_twid_3.c */

/* This file is a slightly modified version of the 
 * file ftw_8.c from the software package FFTW, 
 * version 2.1.1, by M. Frigo and S. G. Johnson.
 * Information about FFTW is available at 
 * http://www.fftw.org
 *
 * The modifications to the FFTW file are that some of
 * the variable names were changed, and the accessing of
 * complex numbers was changed, to accommodate complex 
 * numbers stored in both structs and two-element arrays.  
 * The modifications were made on 6/15/00.
 *
 * The function in this file can be replaced with any 
 * function that computes 
 *
 * x = (F_8 (x) I_<S>) * T^(8*<S>)_<S> * x, 
 *  performed at stride = <iostride>.
 */

#include "spiral_fft.h"

static const fftw_real K707106781 = FFTW_KONST(+0.707106781186547524400844362104849039284835938);

extern void apply_twiddle3( int iostride, Vector(A), fft_value *W, int S, int dist ) 
{
     int i;
     fft_value *inout;

     inout = A;
     for (i = S; i > 0; i = i - 1, inout = at( inout, dist ), 
		 W = at( W, 7 ) ) {
	  fftw_real tmp7;
	  fftw_real tmp43;
	  fftw_real tmp71;
	  fftw_real tmp76;
	  fftw_real tmp41;
	  fftw_real tmp53;
	  fftw_real tmp56;
	  fftw_real tmp65;
	  fftw_real tmp18;
	  fftw_real tmp77;
	  fftw_real tmp46;
	  fftw_real tmp68;
	  fftw_real tmp30;
	  fftw_real tmp48;
	  fftw_real tmp51;
	  fftw_real tmp64;
	  
	  {
	       fftw_real tmp1;
	       fftw_real tmp70;
	       fftw_real tmp6;
	       fftw_real tmp69;
	       
	       tmp1 = c_re(at(inout,0));
	       tmp70 = c_im(at(inout,0));
	       {
		    fftw_real tmp3;
		    fftw_real tmp5;
		    fftw_real tmp2;
		    fftw_real tmp4;
		    
		    tmp3 = c_re(at(inout,4 * iostride));
		    tmp5 = c_im(at(inout,4 * iostride));
		    tmp2 = c_re(at(W,3));
		    tmp4 = c_im(at(W,3));
		    tmp6 = (tmp2 * tmp3) - (tmp4 * tmp5);
		    tmp69 = (tmp4 * tmp3) + (tmp2 * tmp5);
	       }
	       tmp7 = tmp1 + tmp6;
	       tmp43 = tmp1 - tmp6;
	       tmp71 = tmp69 + tmp70;
	       tmp76 = tmp70 - tmp69;
	  }
	  {
	       fftw_real tmp35;
	       fftw_real tmp54;
	       fftw_real tmp40;
	       fftw_real tmp55;
	       
	       {
		    fftw_real tmp32;
		    fftw_real tmp34;
		    fftw_real tmp31;
		    fftw_real tmp33;
		    
		    tmp32 = c_re(at(inout,7 * iostride));
		    tmp34 = c_im(at(inout,7 * iostride));
		    tmp31 = c_re(at(W,6));
		    tmp33 = c_im(at(W,6));
		    tmp35 = (tmp31 * tmp32) - (tmp33 * tmp34);
		    tmp54 = (tmp33 * tmp32) + (tmp31 * tmp34);
	       }
	       {
		    fftw_real tmp37;
		    fftw_real tmp39;
		    fftw_real tmp36;
		    fftw_real tmp38;
		    
		    tmp37 = c_re(at(inout,3 * iostride));
		    tmp39 = c_im(at(inout,3 * iostride));
		    tmp36 = c_re(at(W,2));
		    tmp38 = c_im(at(W,2));
		    tmp40 = (tmp36 * tmp37) - (tmp38 * tmp39);
		    tmp55 = (tmp38 * tmp37) + (tmp36 * tmp39);
	       }
	       tmp41 = tmp35 + tmp40;
	       tmp53 = tmp35 - tmp40;
	       tmp56 = tmp54 - tmp55;
	       tmp65 = tmp54 + tmp55;
	  }
	  {
	       fftw_real tmp12;
	       fftw_real tmp44;
	       fftw_real tmp17;
	       fftw_real tmp45;
	       
	       {
		    fftw_real tmp9;
		    fftw_real tmp11;
		    fftw_real tmp8;
		    fftw_real tmp10;
		    
		    tmp9 = c_re(at(inout,2 * iostride));
		    tmp11 = c_im(at(inout,2 * iostride));
		    tmp8 = c_re(at(W,1));
		    tmp10 = c_im(at(W,1));
		    tmp12 = (tmp8 * tmp9) - (tmp10 * tmp11);
		    tmp44 = (tmp10 * tmp9) + (tmp8 * tmp11);
	       }
	       {
		    fftw_real tmp14;
		    fftw_real tmp16;
		    fftw_real tmp13;
		    fftw_real tmp15;
		    
		    tmp14 = c_re(at(inout,6 * iostride));
		    tmp16 = c_im(at(inout,6 * iostride));
		    tmp13 = c_re(at(W,5));
		    tmp15 = c_im(at(W,5));
		    tmp17 = (tmp13 * tmp14) - (tmp15 * tmp16);
		    tmp45 = (tmp15 * tmp14) + (tmp13 * tmp16);
	       }
	       tmp18 = tmp12 + tmp17;
	       tmp77 = tmp12 - tmp17;
	       tmp46 = tmp44 - tmp45;
	       tmp68 = tmp44 + tmp45;
	  }
	  {
	       fftw_real tmp24;
	       fftw_real tmp49;
	       fftw_real tmp29;
	       fftw_real tmp50;
	       
	       {
		    fftw_real tmp21;
		    fftw_real tmp23;
		    fftw_real tmp20;
		    fftw_real tmp22;
		    
		    tmp21 = c_re(at(inout,iostride));
		    tmp23 = c_im(at(inout,iostride));
		    tmp20 = c_re(at(W,0));
		    tmp22 = c_im(at(W,0));
		    tmp24 = (tmp20 * tmp21) - (tmp22 * tmp23);
		    tmp49 = (tmp22 * tmp21) + (tmp20 * tmp23);
	       }
	       {
		    fftw_real tmp26;
		    fftw_real tmp28;
		    fftw_real tmp25;
		    fftw_real tmp27;
		    
		    tmp26 = c_re(at(inout,5 * iostride));
		    tmp28 = c_im(at(inout,5 * iostride));
		    tmp25 = c_re(at(W,4));
		    tmp27 = c_im(at(W,4));
		    tmp29 = (tmp25 * tmp26) - (tmp27 * tmp28);
		    tmp50 = (tmp27 * tmp26) + (tmp25 * tmp28);
	       }
	       tmp30 = tmp24 + tmp29;
	       tmp48 = tmp24 - tmp29;
	       tmp51 = tmp49 - tmp50;
	       tmp64 = tmp49 + tmp50;
	  }
	  {
	       fftw_real tmp19;
	       fftw_real tmp42;
	       fftw_real tmp63;
	       fftw_real tmp66;
	       
	       tmp19 = tmp7 + tmp18;
	       tmp42 = tmp30 + tmp41;
	       c_re(at(inout,4 * iostride)) = tmp19 - tmp42;
	       c_re(at(inout,0)) = tmp19 + tmp42;
	       {
		    fftw_real tmp73;
		    fftw_real tmp74;
		    fftw_real tmp67;
		    fftw_real tmp72;
		    
		    tmp73 = tmp41 - tmp30;
		    tmp74 = tmp71 - tmp68;
		    c_im(at(inout,2 * iostride)) = tmp73 + tmp74;
		    c_im(at(inout,6 * iostride)) = tmp74 - tmp73;
		    tmp67 = tmp64 + tmp65;
		    tmp72 = tmp68 + tmp71;
		    c_im(at(inout,0)) = tmp67 + tmp72;
		    c_im(at(inout,4 * iostride)) = tmp72 - tmp67;
	       }
	       tmp63 = tmp7 - tmp18;
	       tmp66 = tmp64 - tmp65;
	       c_re(at(inout,6 * iostride)) = tmp63 - tmp66;
	       c_re(at(inout,2 * iostride)) = tmp63 + tmp66;
	       {
		    fftw_real tmp59;
		    fftw_real tmp78;
		    fftw_real tmp62;
		    fftw_real tmp75;
		    fftw_real tmp60;
		    fftw_real tmp61;
		    
		    tmp59 = tmp43 - tmp46;
		    tmp78 = tmp76 - tmp77;
		    tmp60 = tmp51 - tmp48;
		    tmp61 = tmp53 + tmp56;
		    tmp62 = K707106781 * (tmp60 - tmp61);
		    tmp75 = K707106781 * (tmp60 + tmp61);
		    c_re(at(inout,7 * iostride)) = tmp59 - tmp62;
		    c_re(at(inout,3 * iostride)) = tmp59 + tmp62;
		    c_im(at(inout,iostride)) = tmp75 + tmp78;
		    c_im(at(inout,5 * iostride)) = tmp78 - tmp75;
	       }
	       {
		    fftw_real tmp47;
		    fftw_real tmp80;
		    fftw_real tmp58;
		    fftw_real tmp79;
		    fftw_real tmp52;
		    fftw_real tmp57;
		    
		    tmp47 = tmp43 + tmp46;
		    tmp80 = tmp77 + tmp76;
		    tmp52 = tmp48 + tmp51;
		    tmp57 = tmp53 - tmp56;
		    tmp58 = K707106781 * (tmp52 + tmp57);
		    tmp79 = K707106781 * (tmp57 - tmp52);
		    c_re(at(inout,5 * iostride)) = tmp47 - tmp58;
		    c_re(at(inout,iostride)) = tmp47 + tmp58;
		    c_im(at(inout,3 * iostride)) = tmp79 + tmp80;
		    c_im(at(inout,7 * iostride)) = tmp80 - tmp79;
	       }
	  }
     }
}
